from __future__ import absolute_import
from docutils import nodes
import jinja2
from docutils.parsers.rst.directives import unchanged
from docutils.parsers.rst import Directive
import ruamel.yaml

ELEMENT_TEMPLATE = jinja2.Template(
    u"""

<div class="dialogue-element">
    <div class="tab">
      <button class="tablinks" onclick="openSection(event, 'About')" id="defaultOpen">About</button>
      <button class="tablinks" onclick="openSection(event, 'Code')">Code</button>
    </div>
    <div id="About" class="tabcontent">
      {{ description }}
      <h4>Why is this important?</h4>
      {{ motivation }}
      <h4>Examples</h4>
      
      <div class="columns smtb tinycard">
      {% for example in examples %}
        <div class="column">
            <div class="dialogue-example">
              {% for message in example %}
                {% if message.startswith('( ')  %}
                  <div class="chat-bubble user">
                    <p>{{ message | replace('( ', '') }}</p>
                  </div>
                {% elif message.startswith('^ ')  %}
                  <div class="chat-bubble quickreply">
                    <p>{{ message | replace('^ ', '') }}</p>
                  </div>
                {% else %}
                  <div class="chat-bubble bot">
                    <p>{{ message }}</p>
                  </div>
                {% endif %}
              {% endfor %}
            </div>
        </div>
      {% endfor %}
      </div>
      
    </div>
    <div id="Code" class="tabcontent">
      <p>This page contains example code to use as a basis for implementing {{ name }}</p>
      {% if policy is defined and policy is not none %}
        <div>
          To include {{ name }} to your assistant, add the following to the policy definition
          in your configuration file:
          <pre class="copyable-snippet">
            <div class="code copyable" data-clipboard-text="{{policy}}">
{{ policy }}
            </div>
            <button class="button copyable" data-clipboard-text="{{policy}}">copy</button>
          </pre>
        </div>
      {% endif %}
      {% if action is defined and action is not none %}
        <div>
          You can use this custom action definition as a basis for implementing {{ name }}
          <pre class="copyable-snippet">
            <div class="code copyable" data-clipboard-text="{{action}}">
{{ action }}
            </div>
            <button class="button copyable" data-clipboard-text="{{action}}">copy</button>
          </pre>
        </div>
      {% endif %}
      {% if domain is defined and domain is not none %}
        <div>
          Make sure to add this to your domain file to implement {{ name }}
          <pre class="copyable-snippet">
            <div class="code copyable" data-clipboard-text="{{domain}}">
{{ domain }}
            </div>
            <button class="button copyable" data-clipboard-text="{{domain}}">copy</button>
          </pre>
        </div>
      {% endif %}
      {% if nlu is defined and nlu is not none %}
        <div>
          Here is some example NLU data
          {% for intent in nlu %}
            <pre class="copyable-snippet">
              <div class="code copyable" data-clipboard-text="{{intent}}">
{{ intent }}
              </div>
              <button class="button copyable" data-clipboard-text="{{intent}}">copy</button>
            </pre>
          {% endfor %}
        </div>
      {% endif %}
      {% if dialogue is defined and dialogue is not none %}
        <div>
          Here are some example stories for {{ name }}
          {% for story in dialogue %}
            <pre class="copyable-snippet">
              <div class="code copyable" data-clipboard-text="{{story}}">
{{ story }}
              </div>
              <button class="button copyable" data-clipboard-text="{{story}}">copy</button>
            </pre>
          {% endfor %}
        </div>
      {% endif %}
    </div>
</div>
<script type="text/javascript">
  function openSection(evt, sectionName) {
    var i, tabcontent, tablinks;
    tabcontent = document.getElementsByClassName("tabcontent");
    for (i = 0; i < tabcontent.length; i++) {
      tabcontent[i].style.display = "none";
    }
    tablinks = document.getElementsByClassName("tablinks");
    for (i = 0; i < tablinks.length; i++) {
      tablinks[i].className = tablinks[i].className.replace(" active", "");
    }
    document.getElementById(sectionName).style.display = "block";
    evt.currentTarget.className += " active";
  }
  document.getElementById("defaultOpen").click();
</script>

"""
)

# placeholder node for document graph
class element_node(nodes.General, nodes.Element):
    pass


def read_yaml_file(path):
    with open(path, "r") as f:
        string = f.read()
    yaml_parser = ruamel.yaml.YAML(typ="safe")
    yaml_parser.version = "1.1"
    yaml_parser.unicode_supplementary = True

    return yaml_parser.load(string)


class ElementDirective(Directive):
    required_arguments = 0

    option_spec = {"path": unchanged}

    # this will execute when your directive is encountered
    # it will insert a element_node into the document that will
    # get visisted during the build phase
    def run(self):
        env = self.state.document.settings.env
        app = env.app
        node = element_node()
        node["path"] = self.options["path"]
        return [node]


# build phase visitor emits HTML to append to output
def html_visit_element_node(self, node):
    data = read_yaml_file(node["path"])
    html = ELEMENT_TEMPLATE.render(
        description=data["description"],
        motivation=data["motivation"],
        name=data["name"],
        examples=data["examples"],
        nlu=data.get("nlu"),
        dialogue=data.get("dialogue"),
        policy=data.get("policy"),
        domain=data.get("domain"),
        action=data.get("action"),
    )
    self.body.append(html)
    raise nodes.SkipNode


# if you want to be pedantic, define text, latex, manpage visitors too..


def setup(app):
    app.add_node(element_node, html=(html_visit_element_node, None))
    app.add_directive("element", ElementDirective)
